# various nose tests

from nose.tools import *

import resttools.test.test_settings as settings

from resttools.test.irws import IRWS_Test
from resttools.test.nws import NWS_Test
from resttools.test.gws import GWS_Test
